#ifdef EMSCRIPTEN
#include <emscripten.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <SDL.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

struct FontLine { float data[4];int ipts[4]; };
struct FontCircle { float data[5]; int ipts[6]; };

struct FontChar
{ FontLine lines[16]; int numlines; FontCircle circles[16]; int numcircles; };
FontChar font_characters[128];

void initstrfont();

float font_thickness=1.0f;
float font_sharpness=1.0f;
int font_strength=8;

const char fontdata[]="fpaacaimimiiiccagcgekckeeagdglkdklefmfejmjceggkhibinlkfk\
imldfdgcfdggffkhlkljbcflldgghfedjjkimldcfelmkiliicjcfeichcggkiimbaicieabjdjngia\
bhdhnkidaglkfgfkleimicaifilfilibailhnbafilibahmimbalcfmccfjfflfljfflficfjljimda\
fmlmimicicffcblmfmfmkhffkhicbckhihfekhicfkkhimdakekmhcfifilidbfcfgfclcfgkgkgfkk\
laclcfjgefjfilkcbfclcimijijlcjfbchhjhhhjhicghkhimacjilficgmlfjkcaifhfikhkcahfif\
hmikcafijefijmcalgfglkfkcalihelihmccijiiimimfflfjclfiikhbfhklkjhhkfiledehcdedlc\
idlkngnkklemhdaemhciclmgjkjecfmfcfgigfcicfmimimigljicigkeabldllegechmfmfmfcfchc\
lilghclgichmliimeafmfcfclcfmlmfhjhdafmfcfclcfhjhcclklhlhihldehefehlkgldafcfmlml\
cfhlhdaicimgmkmgckccbkckjhckckjekimdafmfcfilchglmcafcfmfmlmeaemececififmcmcmmda\
emececlmlmlcadehllimllkdmgehkdfedbfmfcfcicfjijicijlfbdkkmmehllimllkdmgehkdfeebf\
mfcfcicfjijjjlmicijlfbeggkhlkfkimldfdgcfdggffkhlkljcaimicecmccbecejlcljejljimca\
ecimimmceadcgmgmiiiikmkmnccaecmmemmcdaecihihmcihimdaecmcmcememmmdahbhnhnknhbkbb\
afclmdagbjbjbjnjngncaicfficlfbaemmmbaicjecclgllllmmlhfkggfklkhmdbfmfcimfmifffif\
imljadifimfiimlljmiflgjfdblclmlfiflmimimiffjbdgilihfglfiglllimhflikfcbgmgfehjhg\
fldicbdlllffnlljolggkfggklijkcbfmfclilmfiliifeaimighcicgmkmhgigdbigilighgichcil\
enhndagmgcgilfiilmcbhkhchcgchkkmhldcemefmmmhimihehihgfihmhkfcbfmfflmlilifhifacf\
iliiffjljimdbfoffhfffflhlhfhlifdblolflfjflljljfjlifbbgmgfgilfjfbdfhiilffhgflkfm\
jmiilkjicbhkhcfgkghkkmhlcblflmfffkfklkimcaimffimlfeaefgmgmiiiikmkmmfcafflmfmlfc\
affillfhodafflflffmfmlmcehdhfhlhjjbhdibjnhlinfhhjhihffhhgbaibincejdjfjjjlhbjdib\
jflhjglhjjjijlhninacegiggfigmgkh";

float SampleChar(float px, float py, char cci) {
FontChar& fchr=font_characters[cci]; float mind=font_thickness+1.0f;
for(int i=0;i<fchr.numlines;i++) { float* dp=(float*)&fchr.lines[i];
float ax=dp[0]; float ay=dp[1]; float bx=dp[2]; float by=dp[3];
float dx=bx-ax; float dy=by-ay; float l=sqrtf(dx*dx+dy*dy); float rx=ax;
float ry=ay; if(l>0.0f) { float n=1.0f/l; dx*=n; dy*=n; rx=px-ax; ry=py-ay;
float f=rx*dx+ry*dy; f=f<0.0f?0.0f:(f>l?l:f); rx=ax+dx*f; ry=ay+dy*f; }
rx-=px; ry-=py; float dist=sqrtf(rx*rx+ry*ry); if(dist<mind) mind=dist; }
for(int i=0;i<fchr.numcircles;i++) {
float* dp=(float*)&fchr.circles[i]; float dx=px-dp[0]; float dy=py-dp[1];
float hcf=3.14159265f; float fcf=hcf*2.0f; float a=atan2(-dy, dx)+hcf;
float ras=a-dp[3]; if(ras>hcf) ras=-fcf+ras; else if(ras<-hcf) ras=fcf+ras;
float rs=ras>0.0f?1.0f:-1.0f; ras*=rs; if(ras<dp[4]) a=dp[3]+dp[4]*rs;
float gx=px-(dp[0]-cos(a)*dp[2]); float gy=py-(dp[1]+sin(a)*dp[2]);
float dist=sqrtf(gx*gx+gy*gy); if(dist<mind) mind=dist; } return mind; }

void CircleFromPoints(float* pts, float* center, float* radius){
float c_x=(pts[0]+pts[2])*0.5f; float c_y=(pts[1]+pts[3])*0.5f;
float t_x=c_x-pts[0]; float t_y=c_y-pts[1];
float y0=sqrtf(t_x*t_x+t_y*t_y);
float iy0=1.0f/y0; t_x*=iy0; t_y*=iy0;
float axis_x=t_y; float axis_y=-t_x;
float dx=pts[4]-c_x; float dy=pts[5]-c_y;
float y2=dx*t_x+dy*t_y; float x2=dx*axis_x+dy*axis_y;
float fx=(y2*y2-y0*y0+x2*x2)/(2.0f*x2);
center[0]=c_x+axis_x*fx; center[1]=c_y+axis_y*fx;
dx=pts[0]-center[0]; dy=pts[1]-center[1];
*radius=sqrtf(dx*dx+dy*dy);}

void UpdateCircle(int chri, int ci) {
FontCircle& circle=font_characters[chri].circles[ci];
float fpts[6]; for(int i=0;i<6;i++) fpts[i]=(float)circle.ipts[i];
fpts[4]+=0.1f; fpts[5]+=0.1f;
CircleFromPoints(fpts, &circle.data[0], &circle.data[2]);
float dx=(fpts[0]+fpts[2])*0.5f-circle.data[0];
float dy=(fpts[1]+fpts[3])*0.5f-circle.data[1];
float rx=circle.data[0]-fpts[4]; float ry=circle.data[1]-fpts[5];
if(dx*rx+dy*ry<0.0f) { dx=-dx; dy=-dy; }
float a=atan2(-dy, dx)+3.14159265f; circle.data[3]=a;
dx=fpts[0]-circle.data[0]; dy=fpts[1]-circle.data[1];
a=atan2(-dy, dx)+3.14159265f;
circle.data[4]=fabs(a-circle.data[3]);
if(circle.data[4]>3.14159265f) circle.data[4]=3.14159265f*2.0f-circle.data[4];}

void UpdateLine(int chri, int li){
FontLine& line=font_characters[chri].lines[li];
for(int i=0;i<4;i++) line.data[i]=(float)line.ipts[i];}

int rsv(char*& p) { return (int)(*(p++)-'a'); } void DecodeFont(const char* str){
for(int i=0;i<128;i++) memset(&font_characters[i], 0, sizeof(FontChar));
char *sp=(char*)str; sp=(char*)str; int numchars=rsv(sp)<<4; numchars|=rsv(sp);
for(int i=0;i<numchars;i++){ FontChar& fchr=font_characters[i]; fchr.numlines=rsv(sp); fchr.numcircles=rsv(sp);
for(int j=0;j<fchr.numlines;j++) { for(int k=0;k<4;k++) fchr.lines[j].ipts[k]=rsv(sp); UpdateLine(i, j); }
for(int j=0;j<fchr.numcircles;j++) { for(int k=0;k<6;k++) fchr.circles[j].ipts[k]=rsv(sp); UpdateCircle(i, j); }}}

void initstrfont() { DecodeFont(fontdata); }

////////////////////////////////////////////

int min(int a, int b) { return a<b?a:b; }
int max(int a, int b) { return a>b?a:b; }

SDL_Surface *screen;

float sigmoid(float x)
{
	if(x<0.0f) x=0.0f;
	if(x>1.0f) x=1.0f;
	return 0.5f-cos(x*3.14159265f)/2;
}

int window_width=256;
int window_height=256;
unsigned int* window_buffer=NULL;
void DrawPixel(int ix, int iy, unsigned int color)
{
	if(ix<0 || iy<0 || ix>=window_width || iy>=window_height)
		return;
	window_buffer[iy*window_width+ix]=color;
}
unsigned int ReadPixel(int ix, int iy)
{
	if(ix<0 || iy<0 || ix>=window_width || iy>=window_height)
		return 0x000000;
	return window_buffer[iy*window_width+ix];
}
void DrawBar(int x, int y, int w, int h, unsigned int color)
{
	int x0=max(0, x);
	int y0=max(0, y);
	int x1=min(x+w, window_width);
	int y1=min(y+h, window_height);
	for(int ry=y0;ry<y1;ry++)
		for(int rx=x0;rx<x1;rx++)
			window_buffer[ry*window_width+rx]=color;
}

void DrawChar(int x0, int y0, int sx, int sy, char chri)
{
	if(font_characters[chri].numlines==0 && font_characters[chri].numcircles==0)
		return;
	float xscale=16.0f/sx;
	float yscale=16.0f/sx;
	float aa=font_sharpness*2.5f/(xscale+yscale);
	float thk=1.2f*font_thickness-0.75f/aa;
	for(int y=0;y<sy;y++)
		for(int x=0;x<sx;x++)
		{
			float f=SampleChar((float)x*xscale, (float)y*yscale, chri);
			f-=thk; f=1.0f-f*aa;
			if(f<0.0f) continue; if(f>1.0f) f=1.0f;
			int v=0x55; v-=(int)(v*font_strength/8*f);
			int v0=ReadPixel(x0+x, y0+y)&0xFF;
			if(v>v0) continue;
			DrawPixel(x0+x, y0+y, (v<<16)|(v<<8)|v);
		}
}

void MorphChar(int x0, int y0, int sx, int sy, char chri0, char chri1, float blend)
{
	float xscale=16.0f/sx;
	float yscale=16.0f/sx;
	float aa=font_sharpness*2.5f/(xscale+yscale);
	aa/=1.0f+(0.5f-fabs(blend-0.5f))*2.0f;
	float thk=1.2f*font_thickness-0.75f/aa;
	for(int y=0;y<sy;y++)
		for(int x=0;x<sx;x++)
		{
			float f0=SampleChar((float)x*xscale, (float)y*yscale, chri0);
			float f1=SampleChar((float)x*xscale, (float)y*yscale, chri1);
			float f=f0+(f1-f0)*blend;
			f-=thk; f=1.0f-f*aa;
			if(f<0.0f) continue; if(f>1.0f) f=1.0f;
			int v=0x55; v-=(int)(v*font_strength/8*f);
			int v0=ReadPixel(x0+x, y0+y)&0xFF;
			if(v>v0) continue;
			DrawPixel(x0+x, y0+y, (v<<16)|(v<<8)|v);
		}
}

void PrintString(int x, int y, int sx, int sy, const char* str)
{
	char* sp=(char*)str;
	int x0=x;
	char pc=' ';
	int lfy=sy+2;
	int cfx=sx*3/5+1;
	for(;;)
	{
		char c=*sp++;
		if(c=='\0') return;
		if(c==13 || c==10)
		{
			if(!(c==10 && pc==13)) { x=x0; y+=lfy; }
			continue;
		}
		DrawChar(x, y, sx, sy, c-' ');
		x+=cfx;
	}
}

void GPrint(int x, int y, int s, const char* str, ...)
{
	char temp[1024];
	va_list args;
	va_start(args, str);
	vsprintf(temp, str, args);
	va_end(args);
	PrintString(x, y, s, s, temp);
}

const char words0[]="plane,pilot,car,bag,food,sun,light,break,fun,dance,party,man,chair,plant,apple,lemon,egg,\
storm,rain,air,sky,cloud,grass,sand,tree,leaf,wood,star,one,diet,color,candy,salt,sugar,wet,heavy,time,milk,chess,\
go,warm,cold,late,early,stone,rock,sea,fall,hole,grow,spin,cake,pain,stair,stare,under,over,born,fast,slow,white,\
black,pink,red,oval,round,brick,coin,fire,paper,book,draw,pen,wand,rope,flow,nail,glass,pea,woman,lady,girl,boy,\
ape,cat,dog,mouse,pot,stew,pasta,bread,hair,shoe,sock,tame,tall,short,snow,is,on,at,be,for,was,can,it,yes,no,what,\
sad,pin,go,meet,yet,line,box,fan,pan,tag,ox,yet,say,may,plow,mad,wall,floor,water,in,and,day,night,sword,lance,\
can,cannon,swarm,pool,desk,stick,blob,mast,rake,plume,fog,mist,pot,knife,hook,patch,ghost,eye,ear,tooth,gut,knee,\
hack,leg,toe,print,axe,peak,elf,dire,mark,store,pole,flag,quest,quick,deep,well,ocean,carve,dig,find,hide,scare,\
level,blur,step,loop,pony,hand,thumb,flare,sting,crack,ash,tar,while,void,point,crash,hang,plot,graph,mesh,brain,\
hell,wish,curse,angel,demon,eagle,raven,cow,lion,tiger,stack,belt,snake,wing,zero,mud,damp,moist,pit,boat,ship,\
token,ruby,gold,coal,oak,stray,fine,bloom,stork,crab,squid,snail,note,stove,hit,check,lace,chalk,bag,grape,brush";

int numwords[6];
char wordbuffer[500+400+300+200];
char* wordbuffers[6];

bool word_used[600];

const int gridsize=10;
char gridmap[2*gridsize*gridsize];
char winmap[2*gridsize*gridsize];
bool gridlock[gridsize*gridsize];

int curwl0=0;
int curword0=0;
int curwl1=0;
int curword1=0;
int wordtime=-100;
bool caps=false;

int mouse_x=0;
int mouse_y=0;
bool mouse_left=false;
bool mouse_click=false;

bool dragging=false;
int drag_x0=0;
int drag_y0=0;

char ReadGrid(int b, int x, int y)
{
	if(x<0 || y<0 || x>=gridsize || y>=gridsize) return 'X';
	return gridmap[b*gridsize*gridsize+y*gridsize+x];
}
void WriteGrid(int b, int x, int y, char v)
{
	if(x<0 || y<0 || x>=gridsize || y>=gridsize) return;
	gridmap[b*gridsize*gridsize+y*gridsize+x]=v;
}

int game_level=1;
void initgame();

bool winner()
{
	bool win=true;
	for(int i=0;i<2*gridsize*gridsize;i++)
		if(gridmap[i]!=winmap[i])
		{
			win=false;
			break;
		}
	return win;
}

bool Render()
{
	for(int i=0;i<window_height*window_width;i++)
		window_buffer[i]=0x555555;

	int gx0=128-gridsize/2*22;
	int gy0=128-gridsize/2*22-8;
	int gx=mouse_x-gx0;
	int gy=mouse_y-gy0;
	if(gx>0 && gy>0)
	{
		gx/=22;
		gy/=22;
	}

	unsigned int curscol=0x666666;
	if(dragging)
		curscol=0x888888;
	if(gx>=0 && gx<gridsize && gy>=0 && gy<gridsize)
	{
		if(!gridlock[gy*gridsize+gx])
		{
			DrawBar(gx0+gx*22, gy0+gy*22, 22, 22, curscol);
			if(mouse_click)
			{
				dragging=true;
				drag_x0=gx;
				drag_y0=gy;
			}
			if(dragging && !mouse_left)
			{
				char& c0a=gridmap[drag_y0*gridsize+drag_x0];
				char& c0b=gridmap[gridsize*gridsize+drag_y0*gridsize+drag_x0];
				char& c1a=gridmap[gy*gridsize+gx];
				char& c1b=gridmap[gridsize*gridsize+gy*gridsize+gx];
				char t;
				t=c0a; c0a=c1a; c1a=t;
				t=c0b; c0b=c1b; c1b=t;
			}
		}
	}
	if(!mouse_left)
		dragging=false;

	static float gofs0=0.0f;
	gofs0+=0.02f;
	static float gofs1=0.0f;
	gofs1+=0.013f;
	static float ma=0.0f;
	ma+=0.05f;

	bool win=winner();
	if(win)
	{
		for(int y=0;y<gridsize;y++)
			for(int x=0;x<gridsize;x++)
			{
				float ofs=sin((float)x*1.5f+gofs0)*1.2f + cos(((float)y+(float)x*1.1f)*0.7f-gofs1)*1.4f;
				ofs*=3.3f;
				float f=sin(ma*4.1f+ofs);
				int v=(int)(f*0x20+0x75);
				DrawBar(gx0+x*22, gy0+y*22, 22, 22, (v<<16)|(v<<8)|v);
			}
	}

	for(int y=0;y<gridsize;y++)
		for(int x=0;x<gridsize;x++)
		{
			float ofs=sin((float)x*1.3f+gofs0) + cos(((float)y+(float)x*1.5f)*0.9f-gofs1)*1.2f;
			float f=sin(ma+ofs);
			f=f*0.5f+0.5f;
			f=sigmoid(f);
			if(gridlock[y*gridsize+x])
				font_strength=3;
			else
				font_strength=8;
			MorphChar(gx0-1+x*22, gy0-1+y*22, 24, 24, ReadGrid(0, x, y)-' ', ReadGrid(1, x, y)-' ', f);
		}
	font_strength=8;

	if(dragging)
	{
		int x=drag_x0;
		int y=drag_y0;
		float ofs=sin((float)x*1.3f+gofs0) + cos(((float)y+(float)x*1.5f)*0.9f-gofs1)*1.2f;
		float f=sin(ma+ofs);
		f=f*0.5f+0.5f;
		f=sigmoid(f);
		MorphChar(mouse_x-12, mouse_y-12, 24, 24, ReadGrid(0, x, y)-' ', ReadGrid(1, x, y)-' ', f);
	}

	if(win && mouse_click)
		initgame();

	font_thickness=1.1f;

	int hoverzone=-1;
	if(mouse_y>240)
	{
		hoverzone=mouse_x/64;
	}
	for(int i=0;i<4;i++)
	{
		font_strength=4;
		if(hoverzone==i)
			font_strength=8;
		font_thickness=1.1f;
		font_sharpness=1.0f;
		if(i==game_level)
		{
			font_thickness=1.65f;
			font_sharpness=2.0f;
		}
		GPrint(8+i*64, 243, 10, "LEVEL %i", i+1);
	}
	font_thickness=1.1f;
	font_sharpness=1.0f;
	if(mouse_click && hoverzone!=-1)
	{
		game_level=hoverzone;
		initgame();
	}
	font_strength=8;


	SDL_LockSurface(screen);
	for(int y=0;y<window_height;y++)
	{
		int si=y*window_width;
		int di=y*(screen->pitch/sizeof(unsigned int));
		for(int x=0;x<window_width;x++)
			((unsigned int*)screen->pixels)[di++]=window_buffer[si++];
	}
	SDL_UnlockSurface(screen);
	SDL_UpdateRect(screen, 0, 0, window_width, window_height);

	return true;
}

void Update()
{
	int tick=SDL_GetTicks();

	unsigned int mouse_buttons=SDL_GetMouseState(&mouse_x, &mouse_y);
	bool pl=mouse_left;
	mouse_left=(mouse_buttons&SDL_BUTTON(SDL_BUTTON_LEFT))!=0;
	mouse_click=(mouse_left&&!pl);

	Render();

	SDL_Event event;
	while(SDL_PollEvent(&event)) 
	{
		switch(event.type) 
		{
		case SDL_KEYDOWN:
			break;
		case SDL_KEYUP:
//			if(event.key.keysym.sym == SDLK_ESCAPE)
//				exit(0);
			break;
		case SDL_QUIT:
			exit(0);
		}
	}
}

void initwords()
{
	numwords[5]=0;
	numwords[4]=0;
	numwords[3]=0;
	numwords[2]=0;
	wordbuffers[5]=wordbuffer;
	wordbuffers[4]=wordbuffers[5]+500;
	wordbuffers[3]=wordbuffers[4]+400;
	wordbuffers[2]=wordbuffers[3]+300;
	char* wp=(char*)words0;
	int wl=0;
	char word[8];
	for(;;)
	{
		char c=(*wp++);
		if(c==',' || c=='\0')
		{
			for(int i=0;i<wl;i++)
				wordbuffers[wl][numwords[wl]*wl+i]=word[i];
			numwords[wl]++;
			wl=0;
		}
		else
			word[wl++]=c;
		if(c=='\0')
			break;
	}
}

void initgame()
{
	for(int i=0;i<600;i++)
		word_used[i]=false;
	for(int i=0;i<2*gridsize*gridsize;i++)
		gridmap[i]='+';
	int max_placed=5;
	if(game_level==1)
		max_placed=7;
	if(game_level==2)
		max_placed=10;
	if(game_level==3)
		max_placed=50;
	for(int b=0;b<2;b++)
	{
		int placed_words=0;
		for(int wi=0;wi<125000;wi++)
		{
			int wl=rand()%4+2;
			int ww=rand()%numwords[wl];
			if(word_used[wl*100+ww])
				continue;
			char word[5];
			for(int j=0;j<wl;j++)
				word[j]=wordbuffers[wl][ww*wl+j];
			int dir=rand()%2;
			char dirc1=(dir==0?'H':'V');
			char dirc2=(dir==1?'H':'V');
			int x0=0; int y0=0;
			int dx=0; int dy=0;
			if(dir==0) // H
			{
				x0=rand()%(gridsize+1-wl);
				y0=rand()%gridsize;
				dx=1;
			}
			if(dir==1) // V
			{
				x0=rand()%gridsize;
				y0=rand()%(gridsize+1-wl);
				dy=1;
			}
			bool ok=false;
			int x=x0; int y=y0;
			char c0=ReadGrid(b, x-dx, y-dy);
			if(c0=='+' || c0=='X' || c0==dirc2 || c0==dirc1)
			{
				c0=ReadGrid(b, x+dx*wl, y+dy*wl);
				if(c0=='+' || c0=='X' || c0==dirc2 || c0==dirc1)
				{
					ok=true;
					for(int j=0;j<wl;j++)
					{
						char tc=ReadGrid(b, x, y);
						x+=dx;
						y+=dy;
						if(tc=='+') continue;
						if(tc==word[j]) continue;
						if(tc==dirc1) continue;
						ok=false;
						break;
					}
				}
			}
			if(ok)
			{
				int x=x0; int y=y0;
				if(ReadGrid(b, x-dx, y-dy)>='a' && ReadGrid(b, x-dx, y-dy)<='z')
					WriteGrid(b, x-dx, y-dy, '?');
				else
					WriteGrid(b, x-dx, y-dy, 'Y');
				for(int j=0;j<wl;j++)
				{
					WriteGrid(b, x, y, word[j]);
					if(ReadGrid(b, x-dy, y+dx)=='+')
						WriteGrid(b, x-dy, y+dx, dirc2);
					if(ReadGrid(b, x+dy, y-dx)=='+')
						WriteGrid(b, x+dy, y-dx, dirc2);
					x+=dx;
					y+=dy;
					if(j==wl-1)
					{
						if(ReadGrid(b, x, y)>='a' && ReadGrid(b, x, y)<='z')
							WriteGrid(b, x, y, '!');
						else
							WriteGrid(b, x, y, 'Y');
					}
				}
				word_used[wl*100+ww]=true;
				placed_words++;
				if(placed_words==max_placed)
					break;
			}
		}
	}
	for(int i=0;i<2*gridsize*gridsize;i++)
	{
		char c=gridmap[i];
		if(c=='?' || c=='!')
			continue;
		if(c>='A' && c<='Z')
			gridmap[i]='*';
		if(c==' ')
			gridmap[i]='*';
		if(c=='+')
			gridmap[i]='*';
		if(i<gridsize*gridsize && c>='a' && c<='z')
			gridmap[i]+='A'-'a';
	}
	for(int i=0;i<2*gridsize*gridsize;i++)
	{
		int vi=gridsize*gridsize;
		if(i>=vi)
			vi=-vi;
		if(gridmap[i]=='*' && gridmap[i+vi]=='*')
		{
			gridmap[i]=' ';
			gridmap[i+vi]=' ';
		}
	}
	for(int i=0;i<gridsize*gridsize;i++)
		gridlock[i]=true;
	int nunlock=5;
	if(game_level==1)
		nunlock=8;
	if(game_level==2)
		nunlock=12;
	if(game_level==3)
		nunlock=20;
	while(nunlock>0)
	{
		int x=rand()%gridsize;
		int y=rand()%gridsize;
		int i=y*gridsize+x;
		if(gridlock[i] && gridmap[i]!=' ')
		{
			gridlock[i]=false;
			nunlock--;
		}
	}
	for(int i=0;i<2*gridsize*gridsize;i++)
		winmap[i]=gridmap[i];
	for(int sw=0;sw<5000;sw++)
	{
		int x0=rand()%gridsize;
		int y0=rand()%gridsize;
		int x1=rand()%gridsize;
		int y1=rand()%gridsize;
		if(gridlock[y0*gridsize+x0] || gridlock[y1*gridsize+x1])
			continue;
		char& c0a=gridmap[y0*gridsize+x0];
		char& c0b=gridmap[gridsize*gridsize+y0*gridsize+x0];
		char& c1a=gridmap[y1*gridsize+x1];
		char& c1b=gridmap[gridsize*gridsize+y1*gridsize+x1];
		char t;
		t=c0a; c0a=c1a; c1a=t;
		t=c0b; c0b=c1b; c1b=t;
	}
	if(winner())
		initgame();
}

void Init()
{
	srand(time(NULL));
	initstrfont();
	initwords();
	initgame();
}

int main(int argc, char *argv[])
{
	SDL_Init(SDL_INIT_VIDEO);
	atexit(SDL_Quit);
	screen=SDL_SetVideoMode(window_width, window_height, 32, SDL_SWSURFACE);  
	if(screen==NULL) { exit(1); }
	window_buffer=new unsigned int[window_width*window_height];

	Init();

#ifdef EMSCRIPTEN
	emscripten_set_main_loop(Update, 60, 0);
#else
/*	while (1)
	{
		Update();
		Sleep(16);
	}*/
#endif    

	return 0;
}
